/*
  This program is free software published under the
  terms of the GNU Lesser General Public License.
  http://www.gnu.org/copyleft/lesser.html    
  This program is distributed in the hope that it will
  be useful - WITHOUT ANY WARRANTY; without even the
  implied warranty of MERCHANTABILITY or FITNESS FOR A
  PARTICULAR PURPOSE.
*/ 


/*  xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
     coded in ECMAScript 5 - JavaScript 5  (ES5, ECMAScript 2009)
    xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx */


var changed = false;  // user changed any settings

  
// retrieve the language settings (colors, instructions) from storage if exists  
function init()  
{     
  if (isStorage() && storageAvailable('localStorage'))  {    
    document.getElementById("no_storage").setAttribute("style", "display:none");    
    //document.getElementById("form1").disabled= false;
  }
  else {    
    document.getElementById("no_storage").setAttribute("style", "display:block;font-weight:bold");
    document.getElementById("no_storage").textContent= "Your browser probably doesn't have support for the Web Storage Object or access to the Storage Object is disabled. Check your browser settings. In some browsers (IE11, Edge) Storage Object is not enabled when accessed in a local file (offline). Solution: If you run this application online, this problem should disappear. Or just use another web browser (e.g. Firefox)."; 
    document.getElementById("form1").setAttribute("style", "display:none");
    //document.getElementById("form1").disabled= true; // problem with Edge 
    return;
  }      
	var other = localStorage.getItem('rst-v1.0-other'); // Retrieve the spatial positions (up, down, left, right) and neutral stimulus content from storage          
  if (other)  
	{ 
    other= JSON.parse(other);                            
    for (var i in other)  {  
			var e= document.getElementById("pos" + i);    // spatial positions (up, down, left, right index: 0..3) and neutral stimulus content (index=4)
			e.value= other[i];    
		} 
  }   	
	var clr_def = [ "red", "blue", "green", "yellow", "purple", "brown", "lime", "cyan", "magenta", "silver", "maroon", "olive", "teal", "navy", "pink", "orange" ];    
  var scol = localStorage.getItem('rst-v1.0-colors'); // Retrieve the colors from storage          
  if (scol)  { 
    scol= JSON.parse(scol);                            
    loadColors(scol);
  }
  else  { // load default colors names
    var e, msg;
    for (var i=0; i<16; i++) {
      e= document.getElementById("clr" + i).value= clr_def[i];    // default color name
      msg= document.getElementById("clr" + i + "val").textContent="";  // color validators 
    }      
  }    
  var sins = localStorage.getItem('rst-v1.0-ins'); // Retrieve the instructions text content from storage 
  if (sins) {  
    document.getElementById("ins").value= sins;
    document.getElementById("ins").disabled= false;
    document.getElementById("cins").checked= true;
  } 
  else  {
    document.getElementById("ins").value=""; 
    document.getElementById("ins").disabled= true;
    document.getElementById("cins").checked= false;  
  }    
  if (scol || sins || other)  {
    document.getElementById("del_msg").textContent= "Language settings exists in your web browser."; 
  } 
  document.getElementById("pos0").focus(); 
	//alert("init() is done");
} 


// before using web storage, check browser support for localStorage and sessionStorage
function isStorage()  {
  if (typeof(Storage) !== "undefined") {
     return true;               //  localStorage/sessionStorage is supported     
  } 
  else {
    return false;           // no browser Web Storage support
  }  
}


// necessarily because of IE11 and Edge offline mode
function storageAvailable(type) {
  try  {
    return (window[type]) ? true: false;   
  }
  catch(e) {
    return false;
  }
}


// display the custom color names from browser storage
function loadColors(scol) 
{ 
  for (var i in scol)  
  {  
    var e= document.getElementById("clr" + i);    // color name (input type=text")
    var c= document.getElementById("clr" + i + "c");  // checkbox "Use this color"
    //if (e && c) {
      e.value= scol[i][0];
      c.checked= scol[i][1] ? true: false;
    //}
  } 
}


// delete all language settings from browser local storage
function deleteStore()  
{
  var exists= localStorage.getItem('rst-v1.0-other') || localStorage.getItem('rst-v1.0-colors') || localStorage.getItem('rst-v1.0-ins');                               
  if (!exists)  {
    alert('You have no existing language settings in your web browser to delete');
    return;
  }  
  if ( !window.confirm("Clear existing language settings in your web browser. Continue?") ) { 
    return;
  }                                             
  if (window.confirm("Delete existing language settings in your web browser. Final confirmation?"))  
  {  
    window.localStorage.removeItem("rst-v1.0-other"); 
		window.localStorage.removeItem("rst-v1.0-colors"); 
    window.localStorage.removeItem("rst-v1.0-ins");                
    var s= "All your language settings have been deleted. Language settings are reset to default. Don't forget to refresh the rst.html page.";  
    document.getElementById("del_msg").textContent= s;
    alert(s);
  }          
}  


// user changed any colors 
function isChanged(e)  {  
  changed= true; 
}  


// enable or disable instructions textarea
function enableIns(e) {
  changed= true;
  document.getElementById("ins").disabled= (e.checked) ? false: true;
}  


// save user language settings in browser local storage 
function save()  
{ 
  var error= false, cnt= 0, e, msg, c, ok, is_valid, tmp;  //first,
  is_valid= document.getElementById("is_valid");
  is_valid.textContent="";   
  document.getElementById("feedback").textContent="";
  if (!changed)  {
    is_valid.textContent="You did not change anything. Action aborted."; 
    is_valid.style= "text-align:center;color:#ff0000;font-weight:bold";
    return;
  }      
	
	for (var i=0; i<5; i++)  
  {	// 4 spatial positions + neutral stimulus text content
    e= document.getElementById("pos" + i);    // text field
    msg= document.getElementById("pos" + i + "val");  // validator            
    e.value= sanitize(e.value, true);  
    ok= (e.value.length >= 1 && e.value.length <= 25) ? true: false; 
    if (e.validity.valid && ok)  {     
      msg.textContent="";
    }
    else  {
      msg.textContent= "The allowed text length for Spatial positions and Neutral stimulus text must be in the range: 1-25 characters (range: 1..25).";      
      error= true;
      //first= e;
    }
  }      
	
	for (var i=0; i<16; i++)  
  {  // colors 0..15
    e= document.getElementById("clr" + i);    // color
    msg= document.getElementById("clr" + i + "val");  // color validator        
    if (document.getElementById("clr" + i + "c").checked )  {
      cnt++;
    }
    e.value= sanitize(e.value, true);  
    ok= (e.value.length >= 1 && e.value.length <= 25) ? true: false; // why still validity.valid object is not supported in some browsers? 
    if (e.validity.valid && ok)  {     
      msg.textContent="";
    }
    else  {
      msg.textContent= "The allowed text length for color must be in the range: 1-25 characters (range: 1..25).";      
      error= true;
      //first= e;
    }
  }      
  if (error || cnt != 4) 
  {
    if (cnt != 4) {
      is_valid.textContent="You always have to choose exactly 4 colors. Number of currently selected colors= " + cnt;
    }
    if (error)  {
      is_valid.textContent+=" Some of your language settings are incorrectly set. Correct these settings."; 
    }
    is_valid.style= "text-align:center;color:#ff0000;font-weight:bold";
    //first.focus(); 
    return;
  }
    
  e= document.getElementById("ins");  
  e.value= sanitize(e.value, false); // sanitize instructions
  tmp= e.value.length;
  if ( document.getElementById("cins").checked  && document.getElementById("ins").disabled === false && (tmp < 64 || tmp > 4096) )  { 
		document.getElementById("insval").textContent= "The allowed text length for instruction must be in the range: 64-4096 characters (range: 64..4096). Current length= " + tmp;
		//document.getElementById("insval").textContent= "The minimum length allowed for instruction is 64 characters. The maximum length allowed for instruction is 4096 characters. Current length= " + tmp;
    return;
  } 
  else  {
    document.getElementById("insval").textContent= "";  
  }  
  
  if ( !window.confirm("Save the entered language settings in your web browser?") ) { 
   return;
  } 
  
  // finally update user defined language settings: call window.localStorage.setItem()
  
	var other= [];  // create array for spatial positions + neutral stimulus content (index 4)
  for (var i=0; i<5; i++)    {         
    e= document.getElementById("pos" + i);  // input type="text" id="pos0" .. "pos4"
    //other[i]= [];  
    other[i]= [e.value];  // default other= [ 'up',  'left', 'right', 'down', '????' ] index[4] is reserved for neutral stimulus text content    
  } 

	localStorage.setItem('rst-v1.0-other', JSON.stringify(other));
		
	var scol= [];  // create array for colors
  for (var i=0; i<16; i++)    
  {         
    e= document.getElementById("clr" + i);    // color name (input type=text")
    c= document.getElementById("clr" + i + "c");  // checkbox use this color
    scol[i]= [];  // [color name, use or not] . example: ["red", true] or ["orange", false]
    scol[i]= [e.value, c.checked];     
  }   
 
	localStorage.setItem('rst-v1.0-colors', JSON.stringify(scol));
  
  if (document.getElementById("cins").checked && document.getElementById("ins").disabled === false)  {
    localStorage.setItem('rst-v1.0-ins', document.getElementById("ins").value);
  }
  
  document.getElementById("is_valid").style="text-align:center;color:black";
   
  if (!document.getElementById("cins").checked || document.getElementById("ins").disabled)  {
    document.getElementById("is_valid").textContent+= '  Warning: Instruction text was not saved. You should select "Use this instructions in research" to save.';   
    //alert('Warning: Instruction text was not saved. You should select "Use this instructions in research" to save it.');   
  }      
  changed=false;
  document.getElementById("feedback").textContent= "Your language settings have been successfully updated. Don't forget to refresh the rst.html page.";
  document.getElementById("del_msg").textContent="";
  alert("Your language settings have been successfully updated. Don't forget to refresh the rst.html page.");     
}


// sanitize user input (some functions calls are redundant-backup methods)
function sanitize(s, is_trim) 
{      
  s= s.replace(/&|;|#/g,'');  // remove &;# => see html entities. https://www.w3schools.com/html/html_entities.asp
  s= backslash(s);      
  var e=document.getElementById("tmp");
  e.innerHTML="";
  e.textContent= s;            
  s= e.innerHTML; 
  e.innerHTML="";
	s= strip(s);  
  if (is_trim) { 
    s= trim(s);
  }
  return s;
}

       
// https://stackoverflow.com/questions/19156148/i-want-to-remove-double-quotes-from-a-string
// https://stackoverflow.com/questions/16171320/remove-all-backslashes-in-javascript/46173676
function backslash(s)  {
  s= s.replace(/['"]+/g, '');   // remove single and double qoutes (' and ")
  return s.replace(/\\/g, '')  //  remove backslashes \
}


// https://css-tricks.com/snippets/javascript/strip-html-tags-in-javascript/
function strip(s)  {      
  return s.replace(/(<([^>]+)>)/ig,"");     
}


function trim(str) {
  return str.replace(/^\s\s*/, '').replace(/\s\s*$/, '');
}

 
// http://css-tricks.com/snippets/javascript/htmlentities-for-javascript/
/*function htmlEntities(str) {
  return String(str).replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;').replace(/"/g, '&quot;');
}*/
